//
//  QAPaintableSceneKitView.m
//  TexturePainting
//
//  Created by Daniel Dönigus on 20.10.12.
//  Copyright (c) 2012 Quality Applications. All rights reserved.
//

#import "QAPaintableSceneKitView.h"
#import "QAAppDelegate.h"

@implementation QAPaintableSceneKitView

- (void)mouseDown:(NSEvent *)theEvent {
    QAAppDelegate* delegate = [[NSApplication sharedApplication] delegate];
    
    //Retrieving position of mouse event in view
    CGPoint pointInView = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    //Doing hit test to retrieve object and texture coordinate at mouse position
    NSArray* hitTestResults = [self hitTest:pointInView options: NULL];
    
    if ([hitTestResults count] > 0) {
        self.paintingOnObject = YES;
        //Retrieving the texture coordinate
        CGPoint pointOnObject = [[hitTestResults objectAtIndex:0] textureCoordinatesWithMappingChannel:0];
        //Y-axis is inverted
        pointOnObject.y = 1.0-pointOnObject.y;
        
        //Creating an additional geometry
        [delegate drawingStartedAtPosition:pointOnObject];
    }
}

- (void)mouseUp:(NSEvent *)theEvent {
    QAAppDelegate* delegate = [[NSApplication sharedApplication] delegate];
    
    if (self.paintingOnObject) {
        //finalizing the geometry -> adding to array of geometries
        [delegate finalizeGeometry];
        self.paintingOnObject = NO;
    }
}

- (void)mouseDragged:(NSEvent *)theEvent {
    QAAppDelegate* delegate = [[NSApplication sharedApplication] delegate];
    
    if (self.paintingOnObject) {
        //Retrieving position of mouse event in view
        CGPoint pointInView = [self convertPoint:[theEvent locationInWindow] fromView:nil];
        //Doing hit test to retrieve object and texture coordinate at mouse position
        NSArray* hitTestResults = [self hitTest:pointInView options: NULL];
    
        if ([hitTestResults count] > 0) {
            //Retrieving the texture coordinate
            CGPoint pointOnObject = [[hitTestResults objectAtIndex:0] textureCoordinatesWithMappingChannel:0];
            //Y-axis is inverted
            pointOnObject.y = 1.0-pointOnObject.y;
        
            //Adding a new point to current drawing geometry
            [delegate addAdditionalPointAtPosition:pointOnObject];
        }
    }
}

-(IBAction)moveLeft:(id)sender
{
    //The user pressed the left arrow. Rotating the sphere by 45 degrees. Naturally by an implicit animation!
    SCNNode* geometryNode = [self.scene.rootNode.childNodes objectAtIndex:1];

    [SCNTransaction begin];
    [SCNTransaction setAnimationDuration:2.0];
    geometryNode.rotation = SCNVector4Make(0.0, 1.0, 0.0, M_PI_4);
    [SCNTransaction commit];
}

-(IBAction)moveRight:(id)sender
{
    //The user pressed the left arrow. Rotating the sphere by 45 degrees. Naturally by an implicit animation!
    SCNNode* geometryNode = [self.scene.rootNode.childNodes objectAtIndex:1];

    [SCNTransaction begin];
    [SCNTransaction setAnimationDuration:2.0];
    geometryNode.rotation = SCNVector4Make(0.0, 1.0, 0.0, -M_PI_4);
    [SCNTransaction commit];
}


@end
